/*---------------------------------------------------------------------

generic.cc

This file is part of CytoPacq

Copyright (C) 2007-2013 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: A file containing the definition of methods that creates the general digital phantom.

-------------------------------------------------------------------------*/

#include <string.h>
#include "generic.h"
#include "sim_macros.h"

//------------------------------------------------------------------------
void phantom_generic::read_params(const t_item_seq &phantom_properties,
											 const t_item_seq &common_properties)
{
	 if (common_properties.present("photobleaching"))
	 {
		  photobleaching = common_properties["photobleaching"];
	 }
}

//------------------------------------------------------------------------
phantom_with_position::~phantom_with_position()
{
	 delete position;
}

//------------------------------------------------------------------------
void phantom_with_position::read_params(const t_item_seq &phantom_properties,
													 const t_item_seq &common_properties)
{
	 phantom_generic::read_params(phantom_properties, common_properties);

	 if (phantom_properties.present("position"))
	 { 
		  position = new i3d::Vector3d<float>();
		  t_triplet pos = phantom_properties["position"];

		  position->x = pos.x;
		  position->y = pos.y;
		  position->z = pos.z;
	 };

	 if (common_properties.present("glass slide thickness"))
	 {
		  glass_slide_thickness = common_properties["glass slide thickness"];
	 }

	 if (common_properties.present("influence of gravity"))
	 {
		  const char *text = common_properties["influence of gravity"];

		  if (strcmp(text, "yes") == 0)
				influence_of_gravity = true;
		  else
				influence_of_gravity = false;
	 }

	 if (common_properties.present("cluster effect"))
	 {
		  cluster_effect = common_properties["cluster effect"];
	 }

}

//------------------------------------------------------------------------
