/*---------------------------------------------------------------------

granulocyte.h

This file is part of CytoPacq

Copyright (C) 2007-2013 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: A plugin creating the digital phantom of granulocyte nucleus.

-------------------------------------------------------------------------*/

#ifndef _PHANTOM_GRANULOCYTE_
#define _PHANTOM_GRANULOCYTE_

#include "generic.h"

//------------------------------------------------------------------------

class phantom_granulocyte : public phantom_with_position
{ 
  private:
			 // ---------- internal variables ------------
			 /// the diameter of the whole granulocyte nucleus (in microns)
			 float d_mean;
			 /// the variance of the diameter
			 float d_variance;
			 /// the spread (density) of lobes within the space (number <0;oo))
			 float spread;
			 /// number of lobes (range)
			 t_couple num_of_lobes;
			 /// the lobe size can vary (percents)
			 float lobe_size_variance;
			 /// the lobes sufrace is rather bumpy than smooth (number <0;oo))
			 float lobe_irregularity;
			 /// the width of channels wihci connect the lobes (microns)
			 float snake_width;
			 /// each lobe looks like a hollow egg-shell; this paramter defines 
			 /// the thickness of the shell
			 float shell_thickness;
			 /// the shell volume is not uniform; this parameter defines is non-homogeneity
			 /// (number <0;oo)
			 float chromatin_irregularity;

			 // ---------- internal methods --------------
			 template <class L>
			 bool _create_shape(i3d::Image3d<L> &);

			 template <class T, class L>
			 void _create_texture(const i3d::Image3d<L> &,
										 i3d::Image3d<T> &);

			 template <class T> 
			 void _imitate_chromatin_structure(i3d::Image3d<T> &, 
														  const i3d::Image3d<bool> &);

  public:                               
			 virtual void read_params(const t_item_seq &,
											  const t_item_seq &);

			 virtual bool draw(i3d::Image3d<i3d::GRAY8>&, 
									 i3d::Image3d<LABEL_TYPE>&);

			 virtual bool draw(i3d::Image3d<i3d::GRAY16>&, 
									 i3d::Image3d<LABEL_TYPE>&);

			 /// virtual destructor
			 virtual ~phantom_granulocyte() {};
};

//------------------------------------------------------------------------

#endif
