/*---------------------------------------------------------------------

collisions.h

This file is part of CytoPacq

Copyright (c) 2007-2013 Centre for Biomedical Image Analysis
                        Masaryk University

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda <svoboda@fi.muni.cz>

Description: A simple (correlation based) solution for 
   'object collision problem'.

-------------------------------------------------------------------------*/

#ifndef _COLLISIONS_
#define _COLLISIONS_

#include "i3d/image3d.h"

i3d::Vector3d<float>* 
find_position_uniformly(const i3d::Image3d<bool> &object,
								const i3d::Image3d<bool> &space,
								bool influence_of_gravity);

i3d::Vector3d<float>* 
find_position_in_cluster(const i3d::Image3d<bool> &object,
								 const i3d::Image3d<bool> &space,
								 float cluster_effect,
								 bool influence_of_gravity);

i3d::Vector3d<float>*
find_position_gaussian_weighted(const i3d::Image3d<bool> &object,
										  const i3d::Image3d<bool> &space,
										  i3d::Vector3d<float> gauss_mean,
										  i3d::Vector3d<float> gauss_dev,
										  bool influence_of_gravity);

#endif
