/*---------------------------------------------------------------------

logs.cc

This file is part of CytoPacq

Copyright (C) 2007,2008 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: A module responsible for error logging.

-------------------------------------------------------------------------*/

#include <string>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <time.h>

#include <i3d/basic.h>
#include <i3d/toolbox.h>

#include "sim_macros.h"

using namespace std;

string error_log_name (ERROR_LOG_NAME);
string warning_log_name (WARNING_LOG_NAME);

//----------------------------------------------------------------------

template <class M> void WarningLog(const char* fname, M message)
{
	 ofstream f;
	 time_t result = time(NULL);
	 tm *mtime = localtime(&result);


	 f.open(fname, ios::out | ios::app);
	 f << "Warning ("  <<
				mtime->tm_mday << "." <<
				mtime->tm_mon + 1 << "." <<
				mtime->tm_year + 1900 << " " <<
				mtime->tm_hour << ":" << 
				mtime->tm_min << ":" << 
				mtime->tm_sec << 
				"): " << message << endl;
	 f.close();

#ifdef DEBUG
	 cerr << "*** Warning: " << message << endl << endl;
#endif
}

//----------------------------------------------------------------------

template <class M> void WarningLog(const string& fname, M message)
{
	 WarningLog(fname.c_str(), message);
}

//----------------------------------------------------------------------

template <class M> void ErrorLog(const char* fname, M message)
{
	 ofstream f;
	 time_t result = time(NULL);
	 tm *mtime = localtime(&result);


	 f.open(fname, ios::out | ios::app);
	 f << "Error ("  <<
				mtime->tm_mday << "." <<
				mtime->tm_mon + 1 << "." <<
				mtime->tm_year + 1900 << " " <<
				mtime->tm_hour << ":" << 
				mtime->tm_min << ":" << 
				mtime->tm_sec << 
				"): " << message << endl;
	 f.close();

#ifdef DEBUG
	 cerr << "*** Error: " << message << endl << endl;
#else
	 cerr << "*** Program evaluation has failed. See '" << 
				fname << "'" << endl << endl;
#endif
}

//----------------------------------------------------------------------

template <class M> void ErrorLog(const string& fname, M message)
{
	 ErrorLog(fname.c_str(), message);
}

//----------------------------------------------------------------------

string operator+(const string &fname, int number)
{
	 string path, name, ext;

	 i3d::SplitFilename(fname, path, name, ext);

	 ostringstream my_str;

	 my_str.fill('0');
	 my_str << path << name << "_" << setw(4) << number;
	 my_str << "." << ext;

	 return my_str.str();
}

//------ explicit instantiation ------------------------------------------

template void WarningLog<const char *>(const string &, const char *);
template void WarningLog<const string &>(const string &, const string &);
template void WarningLog<string>(const string &, string);
template void WarningLog<const int>(const string &, const int);
template void WarningLog<i3d::InternalException>(const string &, 
															  i3d::InternalException);
template void WarningLog<i3d::IOException>(const string &, i3d::IOException);

template void ErrorLog<const char *>(const string &, const char *);
template void ErrorLog<const string &>(const string &, const string &);
template void ErrorLog<string>(const string &, string);
template void ErrorLog<const int>(const string &, const int);
template void ErrorLog<i3d::InternalException>(const string &, 
															  i3d::InternalException);
template void ErrorLog<i3d::IOException>(const string &, i3d::IOException);
