/*---------------------------------------------------------------------

illumination.cc

This file is part of CytoPacq

Copyright (C) 2007,2008 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: The simulation of the uneven illumination phenomenon.

Modification: 2012 - Luděk Matyska jr. - proper implementation of uneven illumination

-------------------------------------------------------------------------*/

// standard headers
#include <iostream>

// local headers
#include "illumination.h"

using namespace std;
using namespace i3d;

#define sqr(x) ((x)*(x))

/**************************************************************************/

template <class VOXEL> void AddIllumination(
		Image3d<VOXEL> &img, 
		float shiftx,
		float shifty,
		float *quadp)
{

	echo("\t image offset (microns): " << img.GetOffset());

	// going through every voxel
	for (size_t z=0; z<img.GetSizeZ(); z++)
		for (size_t y=0; y<img.GetSizeY();y++)
			for (size_t x=0; x<img.GetSizeX(); x++)

			{
				// calculating the uneven illuminated value
				float xsf = x + shiftx;
				float ysf = y + shifty;
				float surface = quadp[0]+quadp[1]*xsf+quadp[2]*ysf+quadp[3]*xsf*ysf+quadp[4]*sqr(xsf)+quadp[5]*sqr(ysf);
				float new_value = img.GetVoxel(x,y,z) * surface;

				// store the result
				img.SetVoxel(x,y,z, (VOXEL) new_value);
			}


//	test.SaveImage("test.ics");
}

/**************************************************************************/

/* explicit instantiations */
template  void AddIllumination(
		Image3d<GRAY8> &img, 
		float shiftx,
		float shifty,
		float *quadp);

template  void AddIllumination(
		Image3d<GRAY16> &img, 
		float shiftx,
		float shifty,
		float *quadp);

