#!/bin/bash

# check the number of in arguments
if [ $# != 10 ]; 
then
		  echo "Bad usage of this script.";
		  exit 1;
fi

# This line will be necessary until we update the system on Alfa server
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/usr/local/lib/

# prepare the required variables
DATA_DIR=$1
KEY=$2
WORKING_DIR=$DATA_DIR"/"$KEY

CFG_FILE=$WORKING_DIR"/"$3
PHANTOM_IMG=$WORKING_DIR"/"$4
BLURRED_IMG=$WORKING_DIR"/"$5
BLURRED_PREVIEW=$WORKING_DIR"/"$6
FINAL_IMG=$WORKING_DIR"/"$7
FINAL_PREVIEW=$WORKING_DIR"/"$8
ERROR_LOG=$WORKING_DIR"/"$9
WARNING_LOG=$WORKING_DIR"/"${10}

BIN_PATH=`dirname "$0"`

TO_ADDRESS=`mysql -B -N -e "select email from processes where pid=\"$KEY\""`
VERSION=`mysql -B -N -e "select version from processes where pid=\"$KEY\""`
MAX_SLICE=`mysql -B -N -e "select max_slice from processes where pid=\"$KEY\""`


# ---------------------- start optigen ------------------------------------
# make a record in database
mysql -e "update processes set cytogen_status=\"passed\",optigen_status=\"running\" where pid=\"$KEY\""

# compute
$BIN_PATH/3d-optigen -c $CFG_FILE -p $PHANTOM_IMG -b $BLURRED_IMG -e $ERROR_LOG

if [ $? -ne 0 ];
then
	mysql -e "update processes set optigen_status=\"failed\" where pid=\"$KEY\""
	$BIN_PATH/notify_failed.sh $TO_ADDRESS $KEY $DATA_DIR $VERSION
	exit 1
fi

# in case of success update the database
mysql -e "update processes set optigen_status=\"passed\" where pid=\"$KEY\""

# make preview image
$BIN_PATH/make_preview $BLURRED_IMG $MAX_SLICE $BLURRED_PREVIEW
$BIN_PATH/run-acquigen.sh $1 $2 $3 $4 $5 $6 $7
# ---------------------- stop optigen ------------------------------------

# --------------------- start acquigen ------------------------------------
# make a record in database
mysql -e "update processes set acquigen_status=\"running\" where pid=\"$KEY\""

# compute
$BIN_PATH/3d-acquigen -c $CFG_FILE -b $BLURRED_IMG -f $FINAL_IMG -e $ERROR_LOG

if [ $? -ne 0 ];
then
	mysql -e "update processes set acquigen_status=\"failed\" where pid=\"$KEY\""
	$BIN_PATH/notify_failed.sh $TO_ADDRESS $KEY $DATA_DIR $VERSION
	exit 1
fi

mysql -e "update processes set acquigen_status=\"passed\" where pid=\"$KEY\""

# make preview image
$BIN_PATH/make_preview $FINAL_IMG $MAX_SLICE $FINAL_PREVIEW

# the process has finished successfully
mysql -e "update processes set stop=now() where pid=\"$KEY\""
$BIN_PATH/notify_finished.sh $TO_ADDRESS $KEY $DATA_DIR $VERSION

# --------------------- stop acquigen -------------------------------------
